/*
 * Copyright © 2015 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/idr.h>

#include <linux/kobject.h>
#include <linux/sysfs.h>
#include <backport/bp_module_version.h>

#define DRIVER_AUTHOR	"Tungsten Graphics, Inc."
#define DRIVER_DESC	"Intel Graphics"
#define BUILD_DATE	"20160105"

#ifndef CPTCFG_BASE_KERNEL_NAME
#error "You need a CPTCFG_BASE_KERNEL_NAME"
#endif

#ifndef CPTCFG_DII_KERNEL_HEAD
#error "You need a CPTCFG_DII_KERNEL_HEAD"
#endif

#ifndef CPTCFG_BACKPORTS_RELEASE_TAG
#error "You need a CPTCFG_BACKPORTS_RELEASE_TAG"
#endif

static char *backported_kernel_name = CPTCFG_BASE_KERNEL_NAME;

module_param(backported_kernel_name, charp, 0400);
MODULE_PARM_DESC(backported_kernel_name,
                "The kernel tree name that was used for this backport (" CPTCFG_BASE_KERNEL_NAME ")");

#ifdef BACKPORTS_GIT_TRACKED
static char *backports_tracker_id = BACKPORTS_GIT_TRACKED;
module_param(backports_tracker_id, charp, 0400);
MODULE_PARM_DESC(backports_tracker_id,
                "The version of the tree containing this backport (" BACKPORTS_GIT_TRACKED ")");
#else
static char *backported_kernel_version = CPTCFG_DII_KERNEL_HEAD;
static char *backports_version = CPTCFG_BACKPORTS_RELEASE_TAG;


module_param(backported_kernel_version, charp, 0400);
MODULE_PARM_DESC(backported_kernel_version,
                "The kernel version that was used for this backport (" CPTCFG_DII_KERNEL_HEAD ")");

module_param(backports_version, charp, 0400);
MODULE_PARM_DESC(backports_version,
                "The git version of the backports tree used to generate this backport (" CPTCFG_BACKPORTS_RELEASE_TAG ")");
#endif


struct kobject *kobj_ref;
static ssize_t dkms_i915_show(struct kobject *kobj,
                        struct kobj_attribute *attr, char *buf);
struct kobj_attribute dkms_attr = __ATTR(dkms, 0440, dkms_i915_show, NULL);

/*
** This function will be called when we read the sysfs file
*/
static ssize_t dkms_i915_show(struct kobject *kobj,
                struct kobj_attribute *attr, char *buf)
{
        return sprintf(buf, "%s\n", "dkms_drm");
}

static int __init backport_init(void)
{
	int ret = 0;

	printk(KERN_INFO "I915 COMPAT BACKPORTED INIT\n");
	printk(KERN_INFO "Loading I915 modules backported from " CPTCFG_DII_KERNEL_TAG "\n");
#ifdef BACKPORTS_GIT_TRACKED
       printk(KERN_INFO BACKPORTS_GIT_TRACKED "\n");
#else
       printk(KERN_INFO "I915 Backport generated by backports.git " CPTCFG_BACKPORTS_RELEASE_TAG "\n");

#endif /* BACKPORTS_GIT_TRACKED */
        /*Creating a directory in /sys/kernel/ */
        kobj_ref = kobject_create_and_add("dkms-i915",kernel_kobj);

        /*Creating sysfs file for dkms*/
        ret = sysfs_create_file(kobj_ref,&dkms_attr.attr);
        if (ret) {
                 printk(KERN_ERR "DKMS sysfs create file failed\n");
                 return ret;
        }

	return 0;
}

static void __exit backport_exit(void)
{
	sysfs_remove_file(kernel_kobj, &dkms_attr.attr);
	kobject_put(kobj_ref);
	printk(KERN_INFO "I915 COMPAT BACKPORTED EXIT\n");
}
module_init(backport_init);
module_exit(backport_exit);
MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_VERSION(BACKPORT_MOD_VER);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL and additional rights");
